//--------------------------------------------------------------------------------------
// File: Tutorial01.cpp
//
// Setting up a basic Windows framework including a message loop
//
//--------------------------------------------------------------------------------------
#include <windows.h>

//--------------------------------------------------------------------------------------
// Global Variables
//--------------------------------------------------------------------------------------
HINSTANCE   g_hInst = NULL;
HWND        g_hWnd = NULL;


//--------------------------------------------------------------------------------------
// Forward declarations
//--------------------------------------------------------------------------------------
HRESULT InitWindow( HINSTANCE hInstance, int nCmdShow );
LRESULT CALLBACK    WndProc( HWND, UINT, WPARAM, LPARAM );


//--------------------------------------------------------------------------------------
// Entry point to the program. Initializes everything and goes into a message processing 
// loop. Idle time is used to render the scene.
//--------------------------------------------------------------------------------------
int WINAPI wWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow )
{
	// register and create our window class
    if( FAILED( InitWindow( hInstance, nCmdShow ) ) )
        return 0;

    // main message loop
    MSG msg = {0};
    while( GetMessage( &msg, NULL, 0, 0 ) ) // wait for next message
    {
        TranslateMessage( &msg );
        DispatchMessage( &msg );
    }

    return ( int )msg.wParam;
}


//--------------------------------------------------------------------------------------
// Register class and create window
//--------------------------------------------------------------------------------------
HRESULT InitWindow( HINSTANCE hInstance, int nCmdShow )
{
    // Register class
	WNDCLASSEX wcex;

	// clear out the window class for use
	ZeroMemory(&wcex, sizeof(WNDCLASSEX)); // never forget to zero out, if you do not set all the values!

	// fill the window class struct
    wcex.cbSize			= sizeof( WNDCLASSEX );
    wcex.style			= CS_HREDRAW | CS_VREDRAW; // Redraw when width or height changes
    wcex.lpfnWndProc	= WndProc; // function pointer 
    wcex.hInstance		= hInstance; // Instance handle: important to find window class (CreateWindow)
    wcex.hCursor		= LoadCursor( NULL, IDC_ARROW ); // shape of the cursor
    wcex.hbrBackground	= ( HBRUSH )( COLOR_WINDOW );    // background color
    wcex.lpszClassName	= L"TutorialWindowClass"; // Class name: important to find window class (CreateWindow)

    if( !RegisterClassEx( &wcex ) )
        return E_FAIL;

    // Create window
    g_hInst = hInstance;
    RECT rc = { 0, 0, 640, 480 };
    AdjustWindowRect( &rc, WS_OVERLAPPEDWINDOW, FALSE ); // adjust to match client rect
    g_hWnd = CreateWindow( L"TutorialWindowClass", L"Direct3D 10 Tutorial 0: Setting Up Window", WS_OVERLAPPEDWINDOW,
                           CW_USEDEFAULT, CW_USEDEFAULT, rc.right - rc.left, rc.bottom - rc.top, NULL, NULL, hInstance,
                           NULL );
    if( !g_hWnd )
        return E_FAIL;

    ShowWindow( g_hWnd, nCmdShow );

    return S_OK;
}


//--------------------------------------------------------------------------------------
// Called every time the application receives a message
//--------------------------------------------------------------------------------------
LRESULT CALLBACK WndProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam )
{
    PAINTSTRUCT ps;
    HDC hdc;

    switch( message )
    {
        case WM_PAINT: // Redraw Event
            hdc = BeginPaint( hWnd, &ps ); // get handle to the device context
			// do some windows drawing in here
            EndPaint( hWnd, &ps ); // return the handle to the device contest
            break;

        case WM_DESTROY: // Destroy Event
            PostQuitMessage( 0 );
            break;

        default: // dispatch to default window procedure for the remaining events
            return DefWindowProc( hWnd, message, wParam, lParam );
    }

    return 0;
}

